/* vxconv.c
**
** Hauptprogramm zu `vxconv'.
**
** Aufgabe dieses Programmes ist es, vorhandene Standardfolder der SMTP-Server
** `smail', `sendmail', ... in das VxMail-Format zu konvertieren. Dazu mu
** lediglich eine Indexdatei zu dem jeweiligen Folder erzeugt werden, die die
** Anfangspositionen der einzelnen E-Mails innerhalb des zu konvertierenden
** Folders enthlt.
*/

#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>

#include "config.h"
#include "prefs.h"
#include "conf.h"
#include "tools.h"
#include "vxerror.h"
#include "folder.h"
#include "conv.h"
#include "version.h"
#include "vxopt.h"
#include "inline-funcs.h"

void usage (void)
{
    fprintf (stderr, "\nUsage: %s [-f <configfile>] <folderfile> ..."
		     "\n       %s -h"
		     "\n       %s -V\n"
		     "\n   -f <configfile>"
		     "\n       read configuration from file <configfile>"
		     "\n       (default: %s)"
		     "\n   -h"
		     "\n       show this help"
		     "\n   -V"
		     "\n       show the program's version number\n\n",
		     progname, progname, progname, CONFIGFILE);
    exit (1);
}

int main (int argc, char *argv[])
{
    int ix;
    struct stat dfstat;
    char *folder;
    xFile fx;
    vxopt (&argc, argv, usage);
    reset_umask();
    if ((progname = strrchr(argv[0], '/'))) ++progname; else progname = argv[0];

    if (argc < 2) {
	fprintf (stderr, "%s: no folder file specified!\n", progname);
	exit (1);
    }
    /* Lies die Konfigurationsdatei ein. */
    getconf ();
    get_ug ();
    /* Initialisiere den Konverter. */
    conv_init ();
    /* Konvertiere jede angegebene Folderdatei. */
    for (ix = 1; ix < argc; ++ix) {
	folder = abs_path (prefs.mbox_base, argv[ix]);
#if 0
	if (*argv[ix] == '/') {
	    folder = gdupl (__FILE__, __LINE__, argv[ix]);
	} else {
	    folder = gconc (__FILE__, __LINE__,
			    prefs.mbox_base, "/", argv[ix], NULL);
	}
#endif
	fx = flOpen (argv[ix], "ix", prefs.fperm);
	if (!fx) {
	    fprintf (stderr, "%s: opening folder `%s' failed",
			     progname,
			     folder);
	    perror ("");
	    gfree (folder);
	    continue;
	}
	if (xStat (fx, &dfstat) < 0) {
	    fprintf (stderr, "%s: can't stat open file", progname);
	    perror ("");
	} else if (xChown (fx, dfstat.st_uid, dfstat.st_gid) < 0) {
	    fprintf (stderr, "%s: chown failed", progname);
	    perror ("");
	}
	conv (fx);
	xClose (fx);
	gfree (folder);
    }
    return 0;
}
