/* smtp.c
**
** deliver a message to a non-local recipient
** (through an open SMTP connection).
**
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <stdarg.h>
#include <errno.h>
#include <sys/types.h>

#include "mrmacs.h"
#include "tools.h"
#include "hlsock.h"
#include "inline-funcs.h"
#include "exitcodes.h"
#include "vxerror.h"

#include "userdb.h"

#include "smtp.h"

/* Lies smtliche Zeilen einer SMTP-Statusmeldung ein; gib aber nur den
** Status-Code ser ersten Zeile zurck.
*/
int smtp_result (HLSock serv)
{
    char line[1024], *p;
    int ecode = 0, stv;
    while (noNULL(HLGets (line, 1024, serv))) {
	cuteol (line);
	vxlog (4, "%s\n", line);
	stv = (int) strtol (line, &p, 10);
	if (!ecode) { ecode = stv; }
	if (*p == '\r' || *p == '\n' || *p == ' ') { break; }
    }
    return ecode;
}

int smtp_init (HLSock serv, char *host)
{
    int stv;
    HLhfInit (serv);
    stv = smtp_result (serv);
    if (stv / 100 != 2) { return 1; }
    HLPrint (serv, "HELO %s\r\n", host);
    vxlog (5, ">> HELO %s\n", host);
    stv = smtp_result (serv);
    if (stv / 100 != 2) { return 2; }
    return 0;
}

int smtp_envelope (HLSock serv, char *sender, char *recip)
{
    int stv;
    HLhfInit (serv);
    HLPrint (serv, "MAIL FROM:%s\r\n", sender);
    vxlog (5, ">> MAIL FROM:%s\n", sender);
    stv = smtp_result (serv);
    if (stv / 100 != 2) { return 1; }
    HLPrint (serv, "RCPT TO:%s\r\n", recip);
    vxlog (5, ">> RCPT TO:%s\n", recip);
    stv = smtp_result (serv);
    if (stv / 100 != 2) { return 2; }
    return 0;
}

int smtp_bodyinit (HLSock serv)
{
    int stv;
    HLhfInit (serv);
    HLPuts ("DATA\r\n", serv);
    vxlog (5, ">> DATA\n");
    stv = smtp_result (serv);
    if (stv / 100 != 3) { return 3; }
    return 0;
}

int smtp_send (HLSock serv, char *sender, char *addr, FILE *tfp, off_t from)
{
    int stv, ev, lasteol, waseol;
    char line[1024];

    if (HLhfInit (serv)) {
	vxlog (0, "HLhfInit() failed - %s\n", strerror (errno));
	vxerror_at (EX_OSERR, errno, __FILE__, __LINE__ - 3,
		    "initialising high level functions failed");
    }
    /* Open the files */
    vxlog (5, "seeking position %ld\n", from);
    if (fseek (tfp, from, SEEK_SET) < 0) { return 2; }
    if ((ev = smtp_init (serv, mailhost()))) { return 3; }
    if ((ev = smtp_envelope (serv, sender, addr))) { return 4; }
    if ((ev = smtp_bodyinit (serv))) { return 5; }
    lasteol = 1;
    while (fgets (line, 1024, tfp)) {
	waseol = cuteol (line);
	HLPuts (line, serv);
	vxlog (5, "%s%s", (lasteol ? ">> " : ""), line);
	lasteol = waseol;
	if (waseol) { HLPuts ("\r\n", serv); vxlog (5, "\r\n"); }
    }
    HLPuts (".\r\n", serv);
    vxlog (5, ">> .\r\n");
    stv = smtp_result (serv);
    if (stv / 100 != 2) { return 6; }
    HLPuts ("QUIT\r\n", serv);
    vxlog (5, "QUIT\r\n");
    stv = smtp_result (serv);
    vxlog (5, "mail delivered, QUIT-status = %d\n", stv);
    return (stv / 100 == 2? 0 : 7);
}
