/* tmpfile.c
**
** functions for creating/accessing temporary files
**
*/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>

#include "prefs.h"
#include "tools.h"
#include "vxerror.h"
#include "xfile.h"
#include "mrmacs.h"

#include "tmpfile.h"

/* FILE *fp = fvxtmp()
**     erzeugt eine temporre Datei relativ zu `prefs.tmpdir'. Die Datei
**     wird nach der Erzeugung sofort mit `unlink' entfernt, so da nur
**     der FILE-Zeiger brig bleibt; wird dieser irgendwann geschlossen, dann
**     wird die Datei endgltig gelscht. Zum ffnen der Datei werden drei
**     Versuche mit `fopen' unternommen - mit jeweils verschiedenen Dateinamen.
**     Fhrt keiner dieser Versuche zum Erfolg - weil die Datei bereits
**     existiert, dann wird `errno' auf `EEXIST' gesetzt.
**     Achtung! `fvxtmp' funktioniert nicht auf NFS-Filesystemen.
**     Die Datei kann gelesen werden, wenn sie vorher mittels `rewind' zurck-
**     gesetzt wurde.
*/
FILE *fvxtmp (void)
{
    int fd, ix;
    char buf[60], *tmp;
    pid_t me = getpid();
    FILE *fp;
    for (ix = 0; ix < 3; ++ix) {
	sprintf (buf, "%.8x%.8x", me, rand());
	tmp = gconc (__FILE__, __LINE__,
			   prefs.tmpdir, "/vx_", buf, ".tmp", NULL);
	if ((fd = open (tmp, O_RDWR|O_CREAT|O_EXCL|O_TRUNC, 0600)) < 0) {
	    gfree (tmp);
	    if (errno == EEXIST) { continue; }
	    return (FILE *) NULL;
	}
	ifnull (fp = fdopen (fd, "w+b")) {
	    close (fd); unlink (tmp); gfree (tmp);
	    return (FILE *) NULL;
	}
	unlink (tmp); gfree (tmp);
	return fp;
    }
    errno = EEXIST;
    return (FILE *) NULL;
}

/* xFile xf = xvxtmp()
**     erzeugt ein <folder,index>-Paar temporrer Dateien.
**     Von `xf' kann gelesen werden, wenn es vorher mittels `xReSet' zurck-
**     gesetzt wurde.
*/
xFile xvxtmp (void)
{
    int ix;
    char buf[60], *tmp;
    pid_t me = getpid();
    xFile xf;
    for (ix = 0; ix < 3; ++ix) {
	sprintf (buf, "%.8x%.8x", me, rand());
	tmp = gconc (__FILE__, __LINE__,
		     prefs.tmpdir, "/vx_", buf, ".tmp", NULL);
	if (xCheck (tmp)) { gfree (tmp); continue; }
	ifnull (xf = xOpen (tmp, "wX")) {
	    xUnlink (tmp); gfree (tmp); continue;
	}
	xUnlink (tmp); gfree (tmp);
	return xf;
    }
    errno = EEXIST;
    return (xFile) NULL;
}

/* bertrage eine geffnete Datei (from) auf eine andere, wobei die andere
** Datei vorher geschlossen wird (falls sie geffnet war). (dup2 fr (FILE *))
*/
int fvxtrans (FILE *from, FILE **_to)
{
    FILE *to = *_to;
    int ffd = fileno (from), tfd;
    fpos_t offs;
    ifnull (to) {
	if ((tfd = dup (ffd)) < 0) { return -1; }
	ifnull (to = fdopen (tfd, "w+b")) { close (tfd); return -1; }
	*_to = to;
	return 0;
    } else {
	/* Setze die Dateipuffer zurck; das ganze funktioniert folgendermaen:
	** Wenn die Datei zum schreiben geffnet war, dann werden die noch im
	** Puffer enthaltenen Daten zurckgeschrieben; anschlieend wird der
	** Dateizeiger gelesen und auf die gelesene Position gesetzt. Dadurch
	** wird (hoffentlich) der Puffer geleert.
	*/
	tfd = fileno (to);
	fflush (to); fgetpos (to, &offs); fsetpos (to, &offs);
	fflush (from); fgetpos (from, &offs); fsetpos (from, &offs);
	/* Wenn ich jetzt die Duplikation durchfhre, dann ist es so, als wre
	** `to' gerade eben mit der `fileno' von `from' neu geffnet worden.
	*/
	return (dup2 (ffd, tfd) < 0 ? -1 : 0);
    }
}
